package com.beadlefox.xmlsocket;

/*

Copyright (c) 2007 Jason R. Weiss

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
THE SOFTWARE.

*/


import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

public class RandomEventGenerator extends Thread
{
	//
	// These well-known event names will need to be
	// duplciated in ActionScript so that clients can
	// register for well-known event names
	//
	public static final String EVENT_A = "A";

	public static final String EVENT_B = "B";

	public static final String EVENT_C = "C";

	//
	// Tracks the list of listeners for a given event
	//
	private Map<String, List> map = new HashMap<String, List>();

	//
	// Flag to control event generation loop
	//
	private boolean generateEvents = true;

	public void haltEventGeneration()
	{
		//
		// Generation is to be terminated immediately
		//
		generateEvents = false;
	}

	//
	// Called by a ClientThread to register for events on behalf of a client
	public synchronized void registerEventListener(String eventName,
			RandomEventListener listener)
	{
		if (map.containsKey(eventName))
		{
			//
			// Fetch the list of listeners and append this listener to the
			// notification list
			//
			System.out.println("Registering additional listener for event " + eventName);
			List listeners = map.get(eventName);
			listeners.add(listener);
		} else
		{
			System.out.println("Registering first listener for event " + eventName);
			List listeners = new ArrayList();
			listeners.add(listener);
			map.put(eventName, listeners);
		}
	}

	public void run()
	{
		while (generateEvents)
		{
			//
			// Generate a random event
			//
			int randomEvent = (int) (Math.random() * 10);
			System.out.println("Random Event Generated: " + randomEvent);

			switch (randomEvent)
			{
			case 1: // Event A
				dispatchEvents(EVENT_A);
				break;
			case 2: // Event B
				dispatchEvents(EVENT_B);
				break;
			case 3: // Event C
				dispatchEvents(EVENT_C);
				break;
			case 4: // Event A
				dispatchEvents(EVENT_A);
				break;
			case 5: // Event B
				dispatchEvents(EVENT_B);
				break;
			case 6: // Event C
				dispatchEvents(EVENT_C);
				break;
			default:	// Misfire
				break;
			}

			try
			{
				System.out.println("Random Event Generator Sleeping for 3 seconds");
				Thread.sleep(3000);
			} catch (InterruptedException e)
			{
				//
				// time to wake up
				//
			}
		}
	}
	
	private void dispatchEvents(String eventName)
	{
		if (map.containsKey(eventName))
		{
			//
			// Fetch the list of listeners and append this listener to the
			// notification list
			//
			List listeners = map.get(eventName);
			System.out.println("Registered Listener Count for event " + eventName + ": " + listeners.size());
			
			Iterator iter = listeners.iterator();
			while(iter.hasNext())
			{
				System.out.println("Dispatching Event " + eventName + " to listener");
				RandomEventListener listener = (RandomEventListener) iter.next();
				listener.onEvent(eventName);
			}
		} else
		{
			System.out.println("No registered listeners for event " + eventName);
		}
	}
}
